@********************************************************************
@*   crt0.s v1.1 by Jeff Frohwein                                   *
@********************************************************************
    .TEXT

@  Uncomment the following line to allow resulting
@ ROM to work with flash carts/emulators and MBV2-style
@ multiboot cables. You also need to set the text
@ section in your makefile to 0x2000000.

 .equ MultiBoot_Compatible, 1

@  There are two methods for clearing memory and
@ copying appropriate setup data. The preferred
@ method is GBA DMA copy/clear but some emulators
@ do not accurately do DMA. If you have an inaccurate
@ emulator then comment out the following line.
@ If you are testing on real hardware then uncomment
@ the following line.

@ .equ DMACopyClear, 1

    .GLOBAL     _start
_start:
        .ALIGN
        .CODE 32
    @ Start Vector

rom_header:     b       rom_header_end

    @ Nintendo Logo Character Data (8000004h)
        .byte   0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
        .byte   0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
        .byte   0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
        .byte   0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
        .byte   0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
        .byte   0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
        .byte   0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
        .byte   0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
        .byte   0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
        .byte   0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
        .byte   0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
        .byte   0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
        .byte   0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
        .byte   0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
        .byte   0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
        .byte   0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
        .byte   0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
        .byte   0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
        .byte   0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
        .byte   0x00,0x00,0x00,0x00

    @ Software Title (80000A0h)
        .ascii  "S World 3   "

    @ Initial Code (80000ACh)
        .byte   0x00,0x00,0x00,0x00

    @ Maker Code (80000B0h)
        .byte   0x30,0x31

    @ Fixed Value (80000B2h)
        .byte   0x96

    @ Main Unit Code (80000B3h)
        .byte   0x00

    @ Device Type (80000B4h)
        .byte   0x00

    @ Unused Data (7Byte) (80000B5h)
        .byte   0x00,0x00,0x00,0x00,0x00,0x00,0x00

    @ Software Version No (80000BCh)
        .byte   0x00

    @ Complement Check (80000BDh)
        .byte   0xf0

    @ Checksum (80000BEh)
        .byte   0x00,0x00

rom_header_end:

 .ifdef MultiBoot_Compatible
        b       start_vector

@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@ The following reserved bytes are used if the code is compiled for
@ multiboot mode. It does not hurt anything to leave this header in
@ even if the code is not compiled for multiboot. The GBA BIOS will
@ auto-patch the first two bytes with 0x03 and 0x01, respectively,
@ before running any code.
@
boot_method:
        .byte   0       @ boot method (0=ROM boot, 3=Multiplay boot)
        .byte   0       @ slave # (1=slave#1, 2=slave#2, 3=slave#3)
        .byte   0       @ reserved
        .byte   0       @ reserved
        .word   0       @ reserved
        .word   0       @ reserved
        .word   0       @ reserved
        .word   0       @ reserved
        .word   0       @ reserved
        .word   0       @ reserved
@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
.endif

@--------------------------------------------------------------------
@-                          Reset                                   -
@--------------------------------------------------------------------
    .EXTERN     AgbMain
    .GLOBAL     start_vector
    .ARM                                @ ..or you can use CODE 32 here
    .ALIGN
start_vector:
        mov     r0, #0x12               @ Switch to IRQ Mode
        msr     cpsr, r0
        ldr     sp, sp_irq              @ Set SP_irq
        mov     r0, #0x1f               @ Switch to System Mode
        msr     cpsr, r0
        ldr     sp, sp_usr              @ Set SP_usr

@ Enter Thumb mode
        adr    r0,3f + 1
        bx     r0

        .THUMB                          @ ..or you can use .CODE 16 here
3:

 .ifdef MultiBoot_Compatible

@ *** Multiboot Copy Routine ***
@   Check the Program Counter to see if code is running
@ at 0x2000000 or 0x8000000. If it is running at 0x8000000
@ then copy 256K bytes of it to 0x2000000 and then branch
@ to 0x2000000.
@   The reason for all this is to allow a program to be used
@ "as is" with an flash cart/emulator or with an MBV2-style
@ multiboot cable.
@   NOTE: You can also detect if this ROM is running from
@ 0x2000000 by checking the multiboot header above.

@ Make sure we're in ExWRAM
        mov     r0,pc           @ This instruction is redundant. It could be taken out.
        lsr     r0,#24
        cmp     r0,#2           @ Are we running from External (0x2000000) RAM?
        beq     MBCopyTestX     @ Yes, no need to do a copy

@ We were started in ROM, silly emulators :P
@ so we need to copy to ExWRAM
        mov     r0,#0x08
        lsl     r0,#24          @ r0 = 0x8000000
        mov     r1,#0x02
        lsl     r1,#24          @ r1 = 0x2000000
        mov     r6,r1
        mov     r7,#0x4
        lsl     r7,#16          @ r3 = 0x40000
MBCopyLoop:
        ldmia   r0!,{r2,r3,r4,r5}
        stmia   r1!,{r2,r3,r4,r5}
        sub     r7,#16
        bne     MBCopyLoop
@ Jump to the code to execute
        bx      r6

MBCopyTestX:
.endif

@ Clear Internal RAM to 0x00
        mov     r0,#3
        lsl     r0,#24          @ r0 = 0x3000000
        mov     r1,#0x1f
        lsl     r1,#8
        add     r1,#0xb0        @ r1 = 0x7ec0/4
        bl      ClearMem

@ Clear BSS section to 0x00
@ (Sometimes BSS may be in External RAM)
        ldr     r0,=__bss_start__
        ldr     r1,=_sdata
        sub     r1,r0
        bl      ClearMem

@ DMA copy initialized data (data section) from LMA to VMA (ROM to RAM)
        ldr     r1,=_edata
        ldr     r2,=_sdata
        sub     r3,r1,r2         @ Is there any data to copy?
        beq     CIDSkip          @ no

.ifdef DMACopyClear
        ldr     r0,reg_base
        ldr     r1,=_datalma
        str     r1,[r0,#0x30]    @ Set source address
        str     r2,[r0,#0x34]    @ Set destination address
        lsr     r3,#2            @ r3=(_edata-_sdata)/4
        strh    r3,[r0,#0x38]    @ Set DMA length
        ldr     r1,dma_copy
        strh    r1,[r0,#0x3a]    @ Start DMA
.else
        ldr     r1,=_datalma
CIDLoop:
        ldmia   r1!,{r0}
        stmia   r2!,{r0}
        sub     r3,#4
        bne     CIDLoop
.endif

CIDSkip:

@ Jump to user code
        ldr     r1, =AgbMain
        mov     lr, pc
        bx      r1
        b       start_vector            @ Reset

@ Clear memory to 0x00
@ r0 = Start Address
@ r1 = Length/4

ClearMem:
.ifdef DMACopyClear
        ldr     r2,reg_base
        adr     r3,fill_val
        str     r3,[r2,#0x30]    @ Set source address (fill value)
        str     r0,[r2,#0x34]    @ Set destination address (fill dest address)
        strh    r1,[r2,#0x38]    @ Set DMA length
        ldr     r1,dma_clrb
        strh    r1,[r2,#0x3a]    @ Start DMA
.else
        mov     r2,#0
ClrLoop:
        stmia   r0!,{r2}
        sub     r1,#1
        bne     ClrLoop
.endif
        bx      lr

    .ALIGN
fill_val: .word 0
sp_usr:   .word 0x03008000 - 0x100
sp_irq:   .word 0x03008000 - 0x60
reg_base: .word 0x04000080
dma_clrb: .word 0x8500
dma_copy: .word 0x8400

    .ALIGN
    .CODE 32

    .pool

    .END


