#include "gba.h"
#include "gbalib.h"
#include "m4_gbalib.h"

u16 * draw_screen;

void m4_setframe (u8 framebit)
{
  framebit = framebit &1;

  if (framebit == 1)
    draw_screen = (u16*)0x6000000;
  else
    draw_screen = (u16*)0x600A000;

  framebit = framebit << 4;
  
  REG_DISPCNT = REG_DISPCNT & 0xFFef;
  REG_DISPCNT = REG_DISPCNT + framebit;
}

void m4_hline (u16 x_start, u16 x_end, u16 y, u8 col)
{
  u16 tmp; 
  u16 *ptr;
  u8 c;

  if (x_start & 1)
  {
    u16 t2;
    tmp = draw_screen [(y*120) + (x_start>>1) ]; 
    t2 = col;
    t2 = t2 << 8;
    t2 = t2 + (tmp & 0x00ff);
    draw_screen [(y*120) + (x_start>>1) ] = t2;
    x_start ++;
  }

  if ((x_end & 1) == 0 )
  {
    tmp = draw_screen [(y*120) + (x_end>>1) ]; 
    tmp = tmp & 0xff00;
    tmp = tmp + col;
    draw_screen [(y*120) + (x_end>>1) ] = tmp;
    x_end --;
  }

  // now draw the middle bits of the line, 2 pixels at a time
  tmp = col;
  tmp = tmp << 8;
  tmp += col;

  ptr = &draw_screen[(y*120) +(x_start>>1)];
  for (c = x_start; c <x_end;c+=2)
  {
    *ptr = tmp;
    ptr++;
  }
}

u8 m4_load_pcx (cu8 *pcx_data)
{
	/*Required for decoding*/
	u8 DataByte, HowMany;
	s16 x, y, nextx;
  u16 cnt;

  cnt = 128; 
  x = 0;
  y = 0;

  /*Decode and display graphics*/
  while (y<160) 
  {
		/*Read next byte*/
		DataByte = pcx_data[cnt];
    cnt++;

		/*Reset counter*/
		HowMany=1;

		/*If it is encoded, extract the count information and read in the colour byte*/
		if ((DataByte & 0xC0)==0xC0) 
    {
			HowMany = (DataByte & 0x3F);
  		DataByte = pcx_data[cnt];
      cnt++;
		}

		nextx = x+HowMany;
		if (nextx<240)
		{
			m4_hline(x,nextx,y,DataByte);
		}
		else
		{
			m4_hline(x,240-1,y,DataByte);
			y++;
			nextx -= 240;
			//m4_hline(0,nextx,y,DataByte);
		}
		x=nextx;
  }

  return 1;
}


/*

Text Library Functions

*/

#include "8x16font.h"
#include "fontwidth.h"

u16 m4_letter_pos;
u16 m4_letter_val;
u16 m4_letter_x;

u16 m4_text_color = 255;
u16 m4_text_outline_color = 254;
u16 m4_text_color_h = (255 << 8);
u16 m4_text_outline_color_h = (254 << 8);

void m4_draw_2_pixels_start (u16 x, u16 y)
{
  m4_letter_x = x;
  m4_letter_pos = (y*120) + (x>>1);
  m4_letter_val = draw_screen [m4_letter_pos];
}

void m4_draw_2_pixels (u8 col)
{
  if (!(m4_letter_x & 1))
  {
    if (col == 2)
    {
      m4_letter_val = m4_letter_val & 0xff00;
      m4_letter_val = m4_letter_val + m4_text_outline_color;
    }
    if (col == 3)
    {
      m4_letter_val = m4_letter_val & 0xff00;
      m4_letter_val = m4_letter_val + m4_text_color;
    }
  }
  else
  {
    if (col == 2)
    {
      m4_letter_val = m4_letter_val & 0x00ff;
      m4_letter_val = m4_letter_val + m4_text_outline_color_h;
    }
    if (col == 3)
    {
      m4_letter_val = m4_letter_val & 0x00ff;
      m4_letter_val = m4_letter_val + m4_text_color_h;
    }
    draw_screen [m4_letter_pos] = m4_letter_val;
    m4_letter_pos ++;
    m4_letter_val = draw_screen [m4_letter_pos];
  }
  m4_letter_x++;
}

void m4_letter (u16 x, u16 y, u8 c)
{
  u16 count;
  u16 yp;
  u16 tpos;
  u8 l1,l2;
  u8 t;
  count = c-32;
  count = count*(16*2) ;

  for (yp = y; yp< y+16;yp++)
  {
    l1 = font[count];
    l2 = font[count+1];
    count += 2;

    if (l1 >0 || l2 > 0)
    {
      m4_draw_2_pixels_start (x,yp); 
     
      t = (l1 & 0xc0)>>6;
      m4_draw_2_pixels (t);
      tpos++;
  
      t = (l1 & 0x30)>>4;
      m4_draw_2_pixels (t);
      tpos++;

      t = (l1 & 0x0c)>>2;
      m4_draw_2_pixels (t);
      tpos++;
  
      t = (l1 & 0x03);
      m4_draw_2_pixels (t);
      tpos++;
  
      t = (l2 & 0xc0)>>6;
      m4_draw_2_pixels (t);
      tpos++;
  
      t = (l2 & 0x30)>>4;
      m4_draw_2_pixels (t);
      tpos++;
  
      t = (l2 & 0x0c)>>2;
      m4_draw_2_pixels (t);
      tpos++;
  
      t = (l2 & 0x03);
      m4_draw_2_pixels (t);
      tpos++;
    }
  }
}

void m4_string (u16 x, u16 y, u8 *str)
{
  u16 xp;
  u16 p;
  u8 c;
  xp = x;
  p = 0;
  while (str[p] != 0)
  {
    c = str[p];
    m4_letter (xp, y, c);
    p++;
    xp=xp+fontwidth[c]-1;
  }

}

void m4_string_center (u16 y, u8 *str)
{
  u16 tot_x;
  u8 * s;
  u8 c;
  s = str;
  tot_x = 0;
  while (*s != 0)
  {
    c = s[0];
    s++;
    tot_x=tot_x+fontwidth[c]-1;
  }

  tot_x = 240 - tot_x;
  tot_x = tot_x >> 1;
  m4_string (tot_x , y, str);
}
