/******************************************************************
 * S World 3
 * 
 * By Shen Mansell (shenmansell@hotmail.com
 * http://shen.mansell.tripod.com
 *
 * This program requires GCC to be compiled. Download it from
 * http://www.devrs.com/gba
 *
 * Version 1.0 - 19 May 2001
 *
 ******************************************************************
 */

#include <stdlib.h>
#include "libs/gba.h"
#include "libs/gbalib.h"
#include "sworld3.h"
#include "enemy.h"
#include "misc_actors.h"
#include "main.h"
#include "non_core/menu.h"


#include "images/sprites.h"

#define sprite_off  ((Sprite) {512,0,170,0})
#define sprite_clean ((Sprite) {0,0,0,0})

u16 * tpal;
u16 * tiles;
u16 * map0;
u16 * map1;
u16 * sprite_vram;

u16 last_joy;

void player_fire_a (void);

player_t player;

// how many player shots are active?
u8 num_player_shots = 0;

player_shot_t player_shot[max_player_shots] ;

// did the last bullet go up or down? 
u8 last_bullet = 0;

// how many enemies are active;
u8 num_enemies = 0;

// how many enemies should be on screen at once?
u8 available_enemies = 2;

enemy_t enemy[max_enemies];

u8 num_enemy_shots = 0;

enemy_shot_t enemy_shot[max_enemy_shots];

u8 num_misc_actors = 0;

misc_actor_t misc_actor [max_misc_actors];

void setup_screen (void)
{
  u16 cnt;
 
  tpal  = REG_PALETTE_BASE;           // background palette pointer
  
  //tiles = REG_TILE_BASE + 0x4000;     // tiles pointer
  tiles = (u16 *)0x6000000;

  // clear the video memory
  for (cnt = 0; cnt < 49152; cnt++)
    tiles[cnt] = 0;

  map0  = REG_TILEMAP_BASE;           // tile map pointer
  map1  = REG_TILEMAP_BASE + 0x800;   // tile map for background1

  sprite_vram = REG_SPRITE_BASE;

  // oops magic numbers -
  // 0x80 represents 1 palette of 256 colours
  // 0x04 character set for this plane is stored at TILEBASE + 0x4000
  // 0x200 the tilemap (screen memory) for this plane is stored at TILEMAPBASE + 0x800
  REG_BG0CNT =  0x84;                  // tiles @ TILEBASE+0x4000, tilemap @ TILEMAPBASE
  REG_BG1CNT =  0x284;                 // tiles @ TILEBASE+0x4000, tilemap @ TILEMAPBASE + 0x800
  REG_BG2CNT =  0x87;                  // tiles @ TILEBASE+0x4000, tilemap @ TILEMAPBASE

  REG_DISPCNT = 0 | BG_MODE_1 | OBJ_2D_MAP | OBJ_ENABLE | BG2_ENABLE;
  REG_DISPSTAT = 0;
}

void setup_game_variables (void)
{
  player.score = 0;
  player.lives = 3;
  player.fire = player_fire_a;

  num_player_shots = 0;
  last_bullet = 0;
  available_enemies = 2;
  num_enemies = 0;
  num_enemy_shots = 0;
  num_misc_actors = 0;
  last_joy = 0;
}

void setup_player (void)
{
  player.sprite = sprite_clean;
  sprite_shape (&player.sprite,1);
  sprite_size  (&player.sprite,2);
  sprite_tile  (&player.sprite,0);
  sprite_move (&player.sprite, 0, 80);

  player.power = 1;
  player.fire_param1 = 0;
  player.fire_param2 = 0;
  player.last_fire = 0;
  player.x = 0;
  player.y = 80<<4;
  player.dead = 0;
  player.invincible = 200;
}

void kill_player (void)
{
  player.power = 1;
  put_score (player.score);
  player.dead = 8;
  player.fire_param1 = 0;

  // throw up a few explosions as well 
  create_misc_explosion (player.x, player.y);
  create_misc_explosion (player.x+(12*16), player.y+160);
  create_misc_explosion (player.x+320, player.y+60);
  
}

void setup_sprites (void)
{
  u16 i;

  for (i = 0; i < 256; i++)              // copy colour palettes
    REG_SPRITE_PAL_BASE[i] = sprites_pal[i];

  REG_PALETTE_BASE[0] = 32767;

  sprite_vram = (u16 *) 0x6010000;
  loadchardata(sprites_tiles,sprites_size,&sprite_vram[0]); 
 
	//make sure all sprites are displayed outside visible range
  for (i = 0; i < 128; i++)
	{
    OAM[i] = sprite_off;
  }

  for (i = 0 ; i < max_player_shots; i++)
    player_shot[i].sprite = sprite_off;
  
  for (i = 0 ; i < max_enemies; i++)
    enemy[i].sprite = sprite_off;

  for (i = 0 ; i < max_enemy_shots; i++)
    enemy_shot[i].sprite = sprite_off;

  for (i = 0 ; i < max_misc_actors; i++)
    misc_actor[i].sprite = sprite_off;

  setup_player ();
  
  for (i = 0;i<8;i++)
  {
    OAM[i] = sprite_clean;
    sprite_move (&OAM[i],((i*6)+6),2);
    sprite_tile (&OAM[i],20);
  }

}

void pause (void)
{
  u16 joy;

  joy = (volatile u16) REG_P1;
  joy = -joy;
  while (joy & J_START) 
  {
    vsync();
    joy = (volatile u16) REG_P1;
    joy = -joy;
  }
  while (!(joy & J_START))
  {
    vsync();
    joy = (volatile u16) REG_P1;
    joy = -joy;
  }
  while (joy & J_START) 
  {
    vsync();
    joy = (volatile u16) REG_P1;
    joy = -joy;
  }
}

void put_score (u16 s)
{
  u16 x;
  u16 t;
  
  /*put the score on the screen */  
  x = (s / 10000) % 10;
  t = x;
  t = t+20;
  sprite_tile (&OAM[3],t);
  
  x = (s / 1000) % 10;
  t = x;
  t = t+20;
  sprite_tile (&OAM[4],t);
 
  x = (s / 100)% 10;
  t = x;
  t = t+20;
  sprite_tile (&OAM[5],t);

  x = (s /10) % 10;
  t = x;
  t = t+20;
  sprite_tile (&OAM[6],t);

  x = s % 10;
  t = x;
  t = t+20;
  sprite_tile (&OAM[7],t);
}

void create_player_fire_a (s16 dx, s16 dy)
{
  u8 c = num_player_shots;
  if (num_player_shots < max_player_shots)
  {
    player_shot[c].x = player.x+(28<<4);
    player_shot[c].y = player.y+(9<<4);
    player_shot[c].dx = dx;
    player_shot[c].dy = dy;
    player_shot[c].power = 1;
    player_shot[c].sprite = sprite_clean;
    sprite_size(&player_shot[c].sprite,0);
    sprite_shape(&player_shot[c].sprite,0);
    sprite_tile(&player_shot[c].sprite,4);

    num_player_shots++;
  }
}

void player_fire_a ()
{
  player.last_fire = 8;

  if (player.power ==1)
  {
    create_player_fire_a (64, 0);
  }
  else if (player.power ==2)
  {
    create_player_fire_a (64,0);
    if (last_bullet == 0)
    {
      create_player_fire_a (60,-20);
      last_bullet = 1;
    }
    else
    {
      create_player_fire_a (60,20);
      last_bullet = 0;
    }
  }
  else if (player.power ==3)
  {
    create_player_fire_a (56,-30);
    create_player_fire_a (64,0);
    create_player_fire_a (56,30);
  }
  else if (player.power ==4)
  {
    create_player_fire_a (56,-30);
    create_player_fire_a (60,-10);
    create_player_fire_a (60,10);
    create_player_fire_a (56,30);
  }
  else if (player.power == 5)
  {
    create_player_fire_a (56,-30);
    create_player_fire_a (60,-14);
    create_player_fire_a (64,0);
    create_player_fire_a (60,14);
    create_player_fire_a (56,30);
  }

  
}

const u16 fire_power_b_sprite[5] = {5,6,7,8,9};

void create_player_fire_b (s16 x, s16 y, s16 dx, s16 dy)
{
  u8 c = num_player_shots;
  u16 s;

  if (num_player_shots < max_player_shots)
  {
    player_shot[c].x = x;
    player_shot[c].y = y;
    player_shot[c].dx = dx;
    player_shot[c].dy = dy;
    player_shot[c].power = player.power;

    player_shot[c].sprite = sprite_clean;
    sprite_size(&player_shot[c].sprite,0);
    sprite_shape(&player_shot[c].sprite,0);

    s = fire_power_b_sprite[player.power - 1];
    sprite_tile(&player_shot[c].sprite,s);

    num_player_shots++;
  }

}

void player_fire_b ()
{
  player.last_fire = 2;

  create_player_fire_b (player.x+(28<<4),player.y+(9<<4),48,0);

}


void create_player_fire_c (s16 dx, s16 dy)
{
  u8 c = num_player_shots;
  if (num_player_shots < max_player_shots)
  {
    player_shot[c].x = player.x+(28<<4);
    player_shot[c].y = player.y+(9<<4);
    player_shot[c].dx = dx;
    player_shot[c].dy = dy;
    player_shot[c].power = 1;
  
    player_shot[c].sprite = sprite_clean;
    sprite_size(&player_shot[c].sprite,0);
    sprite_shape(&player_shot[c].sprite,0);
    sprite_tile(&player_shot[c].sprite,4);

    num_player_shots++;
  }
}

void player_fire_c ()
{
  extern const u8 sin_table [360];
  s16 dx,dy;

  if (player.power >1)
    player.last_fire = 4;
  else
    player.last_fire = 8;

  if (player.fire_param2 == 1)
  {
    player.fire_param1 += 25;
    if (player.fire_param1 > 360)
      player.fire_param1 -= 360;
  
    dx = player.fire_param1;
    dy = (dx + 90);
    if (dy > 360)
      dy -= 360;
  
    dx = sin_table[dx];
    dx=dx>>1;
    dx -= 64;
  
    dy = sin_table[dy];
    dy=dy>>1;
    dy -= 64;
    player.fire_param2 = 0;
    create_player_fire_c (dx,dy);
    if (player.power >2)
      create_player_fire_c (-dx,-dy);
    if (player.power >3)
      create_player_fire_c (-dy,dx);
    if (player.power >4)
      create_player_fire_c (dy,-dx);
      
  }
  else
  {
    dx =64;
    dy = 0;
    player.fire_param2 = 1;
    create_player_fire_c (dx,dy);
  }

}

void player_dead_turn (void)
{
  // show some explosions and stuff.
  
  player.y += player.dead;
  player.x += 64;
  sprite_move (&player.sprite, player.x>>4, player.y>>4);
  player.dead += 2;
  player.fire_param1++;
  if (player.fire_param1 == 3)
  {
    create_misc_explosion (player.x, player.y);
    player.fire_param1 = 0;
  }

  if ((player.x > (240*16)) || (player.y > (160*16)))
  {
    player.lives --;
    if (player.lives > 0)
    {
      setup_player ();
    }
  }

}

const u16 powerup_cost [5] = {0,5,10,15,20};

void player_turn (void)
{
  u16 joy;

  if (player.last_fire > 0)
    player.last_fire --;

  if (player.invincible > 0)
  {
    player.invincible --;
    if (player.invincible > 10)
      create_misc_sparkle (player.x + (rand()&511),player.y+(rand()&255));
  }

  if (player.dead > 0)
  {
    player_dead_turn ();
    return;
  }

  joy = (volatile u16) REG_P1;
  joy = -joy;
  if (joy & J_LEFT)
  {
    if (player.x > 0)
      player.x -= 16;
  }
  if (joy & J_RIGHT)
  {
    if (player.x < (200<<4))
      player.x += 16;
  }
  if (joy & J_DOWN)
  {
    if (player.y < (140<<4))
      player.y += 16;
  }
  if (joy & J_UP)
  {
    if (player.y > 0)
      player.y -= 16;
  }
  if ((!(joy & J_A)) )//&& (last_joy & J_A)
  {
    if (player.last_fire == 0)
      player.fire();
  }


  if ((!(joy & J_L)) && (last_joy & J_L))
  {
    if ((player.power < 5) && (player.score >= powerup_cost [player.power]))
    {
      player.score -= powerup_cost [player.power];
      put_score (player.score);
      player.power++;
    }
  }


  if ((!(joy & J_R)) && (last_joy & J_R))
  {
    if (player.score >0)
    {
      player.score --;
      put_score (player.score);  
      if (player.fire == player_fire_a)
        player.fire = player_fire_b;
      else
        player.fire = player_fire_a;
    }
  }
  if ((joy & J_START) && !(last_joy & J_START))
  {
    pause ();
  }

  if (joy & J_SELECT)
  {
    vsync();
    vsync();
    vsync();
  }

  last_joy = joy;
  sprite_move (&player.sprite, player.x>>4, player.y>>4);


}

void player_shot_kill (u8 shot_num)
{
  u8 replace = num_player_shots -1;
 
  if (shot_num < replace)
  {
    player_shot[shot_num] = player_shot[replace];
  }

  player_shot[replace].sprite = sprite_off;
  num_player_shots --;

}

u8 player_shot_turn (u8 shot_num)
{
  player_shot[shot_num].x += player_shot[shot_num].dx;
  player_shot[shot_num].y += player_shot[shot_num].dy;

  // check if the bullet should be turned off
  if ( (player_shot[shot_num].x > (260 << 4)) ||(player_shot[shot_num].x < -(8<<4)) 
     || (player_shot[shot_num].y > (160 << 4)) || (player_shot[shot_num].y < -(8<<4)) )
  {
    player_shot_kill (shot_num);
    sprite_move (&player_shot[shot_num].sprite, player_shot[shot_num].x>>4, player_shot[shot_num].y>>4);
    return 0;
  }
  
  sprite_move (&player_shot[shot_num].sprite, player_shot[shot_num].x>>4, player_shot[shot_num].y>>4);
  return 1;
}

void check_enemy_hit (void)
{
  // incredibly stupid collision detection
  u8 b,e;
  u16 px1, px2, py1,py2;
  b = 0; 
  // for each bullet
  while (b < num_player_shots)
  {
    e = 0;
    px1 = player_shot[b].x;
    px2 = player_shot[b].x + 128;
    py1 = player_shot[b].y;
    py2 = player_shot[b].y + 128;
    // check if it hits each enemy
    while (e < num_enemies)
    {
      if ((enemy[e].dead == 0)
       && (px2 > enemy[e].x  )
       && (px1 < enemy[e].x + enemy[e].size_x )
       && (py2 > enemy[e].y  )
       && (py1 < enemy[e].y + enemy[e].size_y ) 
      )
      {
        if (player_shot[b].power == 1)
          player_shot_kill (b);
        else
        {
          player_shot[b].power = player_shot[b].power -1;
          sprite_tile (&player_shot[b].sprite, player_shot[b].power+4);
        }
        enemy[e].hit_points --;

        if (enemy[e].hit_points == 0)
        {
          enemy[e].sprite = sprite_off;
          enemy[e].dead = 1;
          // only coins give score now
          //player.score += enemy[e].score;
          //put_score (player.score);
        }
        e = num_enemies; // remove this if the bullet should hit as many as possible at once
      }
      else
        e++;
    }
    b++;
  }
}

void check_player_hit (void)
{
  // increadibly stupid collision detection
  u16 px1,py1,px2,py2;
  u8 e;

  if (player.dead > 0) // the player isn't going to get more dead
    return;

  if (player.invincible > 0) // the player can't be hit if they are invincible
    return;

  px1 = player.x+64;
  px2 = player.x+456;
  py1 = player.y+128;
  py2 = player.y+160;
  
  e = 0; 
  // for each enemy
  while (e < num_enemies)
  {
    // check if it hits the player
    if ((enemy[e].dead == 0)
     && (px1 < enemy[e].x+enemy[e].size_x)
     && (px2 > enemy[e].x)
     && (py1 < enemy[e].y+enemy[e].size_y)
     && (py2 > enemy[e].y)
        
    )
    {
      kill_player ();
      return;
    }
    e++;
  }

  e = 0; 
  // for each enemy bullet
  while (e < num_enemy_shots)
  {
    // check if it hits the player
    if (
        (px1 < enemy_shot[e].x+128 )
     && (px2 > enemy_shot[e].x)
     && (py1 < enemy_shot[e].y+128 )
     && (py2 > enemy_shot[e].y)
        
    )
    {
      kill_player ();
      return;
    }
    e++;
  }

}

void everyones_turn (void)
{
  u8 c;
  static u16 timer = 0;

  timer ++;
  if (timer>1000)
  {
    timer = 0;
    if (available_enemies < 20)
      available_enemies ++;
  }

  player_turn();
  
  c = 0;
  while (c<num_player_shots)
    c = c + player_shot_turn (c);
    
  c = 0;
  while (c<num_enemies)
    c = c + enemy[c].turn (c);

  c = 0;
  while (c<num_enemy_shots)
    c = c + enemy_shot_turn (c);

  c = 0;
  while (c<num_misc_actors)
    c = c + misc_actors_turn (c);

  check_enemy_hit ();
  check_player_hit();
}

void draw_all_sprites (void)
{
  u8 c;
 
  OAM[player_sprite] = player.sprite;
 
  for (c=0; c<max_player_shots;c++)
  {
    OAM[c+player_shots_sprite_start] = player_shot [c].sprite;
  }

  for (c=0; c<max_enemies;c++)
  {
    OAM[c+enemy_sprite_start] = enemy[c].sprite;
  }

  for (c=0; c<max_enemy_shots;c++)
  {
    OAM[c+enemy_shots_sprite_start] = enemy_shot[c].sprite;
  }

  for (c=0; c<max_misc_actors;c++)
  {
    OAM[c+misc_actors_sprite_start] = misc_actor[c].sprite;
  }
}

void play_game (void)
{
  setup_game_variables ();

  while (player.lives > 0)
  { 
    vsync();
    draw_all_sprites();
    enemy_housekeeping();
    everyones_turn();
  }
}

void AgbMain(void) {
  
  init_gbalib ();

  show_skull ();

  while (1==1)
  {
    show_title ();
    show_instructions ();
    setup_screen ();
    setup_sprites ();
    play_game ();
  }

  
}
