#include <stdlib.h>

#include "libs/gba.h"
#include "libs/gbalib.h"
#include "sworld3.h"
#include "main.h"

#define sprite_off  ((Sprite) {512,0,170,0})
#define sprite_clean ((Sprite) {0,0,0,0})

extern player_t player;

extern u8 num_misc_actors;
extern misc_actor_t misc_actor [max_misc_actors];

void misc_actor_kill (u8 actor_num)
{
  u8 replace = num_misc_actors -1;
 
  if (actor_num < replace)
  {
    misc_actor[actor_num] = misc_actor[replace];
  }

  misc_actor[replace].sprite = sprite_off;
  num_misc_actors --;

}

void create_misc_sparkle (s16 x, s16 y)
{
  u8 c = num_misc_actors;
  if (num_misc_actors < max_misc_actors)
  {
    misc_actor[c].x = x;
    misc_actor[c].y = y;
    misc_actor[c].count = 6;
    misc_actor[c].sprite = sprite_clean;
    if (y&16)
    {
      sprite_tile(&misc_actor[c].sprite,38);
    }
    else
    {
      sprite_tile(&misc_actor[c].sprite,39);
    }

    misc_actor[c].type = misc_sparkle;

    sprite_size(&misc_actor[c].sprite,0);
    sprite_shape(&misc_actor[c].sprite,0);
    sprite_move (&misc_actor[c].sprite, misc_actor[c].x>>4, misc_actor[c].y>>4);
    
    num_misc_actors++;
  }

}

u8 misc_sparkle_turn (u8 actor_num)
{
  misc_actor[actor_num].count --;

  if (misc_actor[actor_num].count == 0)
  {
    misc_actor_kill (actor_num);
    return 0;
  }
  
  return 1;
}



void create_misc_explosion (s16 x, s16 y)
{
  u8 c = num_misc_actors;
  if (num_misc_actors < max_misc_actors)
  {
    misc_actor[c].x = x;
    misc_actor[c].y = y;
    misc_actor[c].count = 10;
    misc_actor[c].type = misc_explosion;

    misc_actor[c].sprite = sprite_clean;
    sprite_size(&misc_actor[c].sprite,1);
    sprite_shape(&misc_actor[c].sprite,0);
    sprite_tile(&misc_actor[c].sprite,64);
    sprite_move (&misc_actor[c].sprite, misc_actor[c].x>>4, misc_actor[c].y>>4);
    
    num_misc_actors++;
  }

}

const u8 misc_explosion_tile [] = {72,72,70,70,68,68,66,66,64,64,64};
u8 misc_explosion_turn (u8 actor_num)
{
  misc_actor[actor_num].count --;

  sprite_tile(&misc_actor[actor_num].sprite,
    misc_explosion_tile[misc_actor[actor_num].count]);

  if (misc_actor[actor_num].count == 0)
  {
    misc_actor_kill (actor_num);
    return 0;
  }
  else
    return 1;
}

void create_misc_coin (s16 x, s16 y, s16 dx, s16 dy)
{
  u8 c = num_misc_actors;
  if (num_misc_actors < max_misc_actors)
  {
    misc_actor[c].x = x;
    misc_actor[c].y = y;
    misc_actor[c].dx = 30;
    misc_actor[c].dy = 0;
    misc_actor[c].count = 0;
    misc_actor[c].type = misc_coin;

    misc_actor[c].sprite = sprite_clean;
    sprite_size(&misc_actor[c].sprite,0);
    sprite_shape(&misc_actor[c].sprite,0);
    sprite_tile(&misc_actor[c].sprite,40);
    sprite_move (&misc_actor[c].sprite, misc_actor[c].x>>4, misc_actor[c].y>>4);
    
    num_misc_actors++;
  }

}

const u8 misc_coin_tile [] = {40,41,42,43,40};

u8 misc_coin_turn (u8 actor_num)
{
  misc_actor[actor_num].count ++;

  // animate the damn thing
  if ((misc_actor[actor_num].count & 7) == 0)
  {
    sprite_tile(&misc_actor[actor_num].sprite,
      misc_coin_tile[misc_actor[actor_num].count>>3]);
    if (misc_actor[actor_num].count == 32)
      misc_actor[actor_num].count = 0;
  }      

  // move the damn thing
  misc_actor[actor_num].dx --;
  misc_actor[actor_num].x += misc_actor[actor_num].dx>>1;

  //misc_actor[actor_num].dy ++;
  //misc_actor[actor_num].y += misc_actor[actor_num].dy>>1;
  //misc_actor[actor_num].x += misc_actor[actor_num].dx;
  
  sprite_move (&misc_actor[actor_num].sprite, 
    misc_actor[actor_num].x>>4, misc_actor[actor_num].y>>4);
  
  // kill it if it is off the screen
  if (misc_actor[actor_num].x < -128)
  {
    misc_actor_kill (actor_num);
    return 0;
  }
  
  // kill it if it is touching the player (and give the player some points)
  if ( (player.x < misc_actor[actor_num].x + 128)
    && (player.y < misc_actor[actor_num].y + 128)
    && (player.x +512 > misc_actor[actor_num].x)
    && (player.y + 256 > misc_actor[actor_num].y) )
  {
    create_misc_sparkle (misc_actor[actor_num].x, misc_actor[actor_num].y);
    misc_actor_kill (actor_num);
    player.score ++;
    put_score (player.score);
  }

  return 1;
}


u8 misc_actors_turn (u8 actor_num)
{
  if (misc_actor[actor_num].type == misc_explosion)
    return misc_explosion_turn (actor_num);

  if (misc_actor[actor_num].type == misc_sparkle)
    return misc_sparkle_turn (actor_num);

  if (misc_actor[actor_num].type == misc_coin)
    return misc_coin_turn (actor_num);

  return 1;
}

